@@ -48,6 +48,8 @@ int width, height;
 
 int quiet = 0;
 int cleanup_on_sane_exit = 1;
+int want_mapping = 1;
+u_int32_t *mapping = NULL;
 
 void
 assert(int bool, char *string)
@@ -128,19 +130,29 @@ void
 resize_video(void)
 {
 
-	if (vidbuf != NULL)
-		free(vidbuf);
+	if (want_mapping) {
+		if (vidbuf != NULL)
+			vid_unmap(vidbuf, vidbuflen);
+	} else {
+		if (vidbuf != NULL)
+			free(vidbuf);
+	}
+	vidbuf = NULL;
 
 	width = aa_imgwidth(context);
 	height = aa_imgheight(context);
 	vidbuflen = width * height * sizeof(u_int32_t);
 
-	vidbuf = (u_int32_t *) malloc(vidbuflen);
-	assert(vidbuf != NULL, "malloc of vidbuf");
-	bzero(vidbuf, vidbuflen);
+	if (want_mapping) {
+		vidbuf = vid_map(vidbuflen);
+		assert(vidbuf != NULL, "vid_map of vidbuf");
+	} else {
+		vidbuf = (u_int32_t *) malloc(vidbuflen);
+		assert(vidbuf != NULL, "malloc of vidbuf");
+		bzero(vidbuf, vidbuflen);
+	}
 
 	assert(vid_setcapturesize(width, height) == 0, "vid_setcapturesize");
-
 	if (!quiet)
 		printf("x: %d y: %d buflen: %d\n", width, height, vidbuflen);
 }
@@ -179,7 +191,6 @@ main(int argc, char *argv[])
 	contrast = 128;
 	channel = 1;
 
-
 	assert(vid_setinputsource(source) == 0, "Failed to set input source");
 	assert(vid_setinputformat(format) == 0, "Failed to set input format");
 	assert(vid_setbrightness(bright) == 0, "Failed to set brightness");
@@ -191,8 +202,13 @@ main(int argc, char *argv[])
 	framemax = -1;
 
 	while (1) {
-		len = vid_read((char *)vidbuf, vidbuflen);
-		assert(len == vidbuflen, "vid_read returned wrong len");
+		if (want_mapping) {
+			assert(vid_waitframe() == 0, "vid_waitframe failed");
+		} else {
+			len = vid_read((char *)vidbuf, vidbuflen);
+			assert(len == vidbuflen,
+			    "vid_read returned wrong len");
+		}
 
 		mean = 0;
 		for (x = 0; x < width; x++) {
